#!/bin/bash
#
# simple script for checking all publicly available records
# if their landing pages are okay
#
# requires `curl` and `jq` to be installed

set -eu

base_url="${BASE_URL:-https://test.researchdata.tuwien.ac.at}"
check_ssl=1

function usage() {
    echo "usage: $0 [-k] [ BASE_URL ]"
    echo
    echo "  -k      Disable SSL verification"
}

function curl() {
    if [[ "${check_ssl}" -gt 0 ]]; then
        command curl "$@"
    else
        command curl -k "$@"
    fi
}

# parse arguments
while getopts "hk" opt; do
    case "${opt}" in
        h)
            usage
            exit 0
            ;;
        k)
            check_ssl=0
            ;;
        *)
            usage >&2
            exit 1
            ;;
    esac
done
shift $(( OPTIND - 1 ))

if [[ $# -gt 0 ]]; then
    base_url="$1"
fi

# prefix the base url with "https://" unless http/https is specified
if [[ ! "${base_url}" =~ ^https?:// ]]; then
    base_url="https://${base_url}"
fi

# fetch the list of records and bail if it doesn't work out
if ! response=$(curl -s --fail "${base_url}/api/records?size=100"); then
    echo "ERROR: couldn't fetch list of records from ${base_url}" >&2
    exit 1
fi
num_hits=$(echo "${response}" | jq .hits.total)

num_ok=0
num_fail=0
num_total=0

offset=0
for idx in $(seq "${num_hits}"); do
    echo -ne "\r${idx}/${num_hits}... "
    [[ ${idx} -eq ${num_hits} ]] && echo "done."

    # take pagination into account for the index
    response_idx=$(( idx - offset - 1 ))

    # check if we need to fetch the next page of results
    if [[ $(echo "${response}" | jq -r ".hits.hits[${response_idx}]") = "null" ]]; then
        next_url=$(echo "${response}" | jq -r ".links.next")
        response=$(curl -s "${next_url}")
        offset=$(( idx - 1 ))
        response_idx=0
    fi

    # get the record's PID and landing page URL
    pid=$(echo "${response}" | jq -r ".hits.hits[${response_idx}].id")
    landing_page_url=$(echo "${response}" | jq -r ".hits.hits[${response_idx}].links.self_html")

    # check the landing page
    if curl -s --fail "${landing_page_url}" > /dev/null; then
        num_ok=$(( num_ok + 1 ))
    else
        echo "FAIL: ${pid} (${landing_page_url})" >&2
        num_fail=$(( num_fail + 1 ))
    fi
    num_total=$(( num_total + 1 ))
done

echo "${num_ok} out of ${num_total} records are okay (${num_fail} failed)"
exit "${num_fail}"
