#!/usr/bin/env python3

import argparse
import re
import subprocess as sp
import sys
from collections import defaultdict

import pendulum

parser = argparse.ArgumentParser()
parser.add_argument(
    "container_name",
    help="the name of the (web-ui) docker container whose logs to analyze",
    nargs="?",
)
parser.add_argument(
    "--read-stdin",
    "-r",
    action="store_true",
    default=False,
    help="read from stdin instead of from docker logs",
)
parser.add_argument(
    "--starting-timestamp", "-t", help="timestamp at which to start reading"
)
parser.add_argument(
    "--ending-timestamp", "-T", help="timestamp at which to stop reading"
)
parser.add_argument(
    "--verbose",
    "-v",
    action="store_true",
    default=False,
    help="print additional details in the summary",
)
args = parser.parse_args()
container_name = args.container_name
read_stdin = args.read_stdin
timestamp = args.starting_timestamp
end_timestamp = args.ending_timestamp
verbose = args.verbose

if timestamp:
    timestamp = pendulum.parse(timestamp, strict=False)

if end_timestamp:
    end_timestamp = pendulum.parse(end_timestamp, strict=False)

if read_stdin:
    lines = sys.stdin

elif container_name:
    cp = sp.run(["docker", "logs", container_name], capture_output=True, text=True)
    lines = (cp.stdout + cp.stderr).splitlines()

else:
    print("error: must read from either docker logs or stdin", file=sys.stderr)
    sys.exit(1)


# pattern for the uwsgi log timestamp format, e.g. [Fri Mar  5 14:37:24 2021]
p_timestamp = re.compile(
    r"(Mon|Tue|Wed|Thu|Fri|Sat|Sun)\s+((Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)[0-9\s:]+)"
)
p_landing_page = re.compile(r"GET /records/(\w+\-\w+)\s")
p_export = re.compile(r"GET /records/(\w+\-\w+)/export/")
p_file = re.compile(r"GET /records/(\w+\-\w+)/files/(.*?)(\?download=1|\s)")


def check_timestamp(line, skip_check=False):
    global timestamp

    if match := p_timestamp.search(line):
        ts = pendulum.parse(match.group(2), strict=False)
        if skip_check or timestamp is None or ts > timestamp:
            if end_timestamp is None or ts < end_timestamp:
                timestamp = ts
                return True

    return False


records = defaultdict(
    lambda: {
        "visits": 0,
        "files": defaultdict(lambda: {"started": 0, "aborted": 0}),
        "exports": 0,
    }
)
skip_check = False
start_date = None

for line in lines:
    line = line.strip()
    if check_timestamp(line, skip_check=skip_check):
        # only use the timestamp check to determine the entry point
        skip_check = True
        if m := p_export.search(line):
            recid = m.group(1)
            records[recid]["exports"] += 1
        elif m := p_file.search(line):
            recid = m.group(1)
            fname = m.group(2)
            if "connection reset by peer" in line.lower():
                records[recid]["files"][fname]["aborted"] += 1
            else:
                records[recid]["files"][fname]["started"] += 1
        elif m := p_landing_page.search(line):
            recid = m.group(1)
            records[recid]["visits"] += 1

        if start_date is None:
            start_date = timestamp

result = dict(records)
for recid in result:
    result[recid]["files"] = dict(result[recid]["files"])

start_heading = f"start date: {start_date.isoformat()}"
end_heading = f"end date:   {timestamp.isoformat()}"
print(start_heading)
print(end_heading)
print("-" * max(len(start_heading), len(end_heading)))

if not result:
    print("nothing to report...")

for recid in result:
    print()
    num_visits = result[recid]["visits"]
    num_exports = result[recid]["exports"]
    details = f"({num_visits} landing page, {num_exports} exports)" if verbose else ""
    print(f"{recid}: {num_visits + num_exports} visits {details}".rstrip())
    for file in result[recid]["files"]:
        num_started = result[recid]["files"][file]["started"]
        num_aborted = result[recid]["files"][file]["aborted"]
        details = f"({num_started} started, {num_aborted} aborted)" if verbose else ""
        print(f"    {file}: {num_started - num_aborted} downloads {details}".rstrip())
