package main

import (
	"bufio"
	"fmt"
	"net/url"
	"os"
	"regexp"
	"sort"
	"strings"
	"time"
)

func increaseCounter(dict map[string]int, text string) {
	if val, ok := dict[text]; ok {
		dict[text] = val + 1
	} else {
		dict[text] = 1
	}
}

func decreaseCounter(dict map[string]int, text string) {
	if val, ok := dict[text]; ok {
		dict[text] = val - 1
	} else {
		dict[text] = -1
	}
}

//
func sortedKeys(dict map[string]int) []string {
	keys := make([]string, len(dict))
	i := 0
	for k := range dict {
		keys[i] = k
		i++
	}
	sort.Strings(keys)
	return keys
}

func main() {
	fileDownloadRegex := regexp.MustCompile(`GET /records/(\w+\-\w+)/files/(.*?)(\?download=1|\s)`)
	exportRegex := regexp.MustCompile(`GET /records/(\w+\-\w+)/export/`)
	landingPageRegex := regexp.MustCompile(`GET /records/(\w+\-\w+)\s`)
	timestampRegex := regexp.MustCompile(`\[(\d{2}/\w{3}/\d{4}:\d{2}:\d{2}:\d{2}\s+.?\d{0,4})\]`)

	scanner := bufio.NewScanner(os.Stdin)
	downloads := make(map[string]int)
	exports := make(map[string]int)
	visits := make(map[string]int)
	var startDate, endDate time.Time

	// read every line from stdin
	for scanner.Scan() {
		line := scanner.Text()

		// the common log format (https://en.wikipedia.org/wiki/Common_Log_Format)
		// specifies the timestamp to have the format '%d/%b/%Y:%H:%M:%S %z'
		// which go wants to have 'by example' (of a specific pre-defined date)
		if match := timestampRegex.FindStringSubmatch(line); match != nil {
			timestamp, err := time.Parse("02/Jan/2006:15:04:05 -0700", match[1])
			if err == nil {
				endDate = timestamp
				if startDate.IsZero() {
					startDate = timestamp
				}
			}
		}

		// parse the line and increase the appropriate counter
		if landingPageRegex.MatchString(line) {
			recid := landingPageRegex.FindStringSubmatch(line)[1]
			increaseCounter(visits, recid)

		} else if fileDownloadRegex.MatchString(line) {
			submatch := fileDownloadRegex.FindStringSubmatch(line)
			recid := submatch[1]
			file := submatch[2]
			name := recid + "/" + file

			// check if the log indicates a download or error
			if strings.Contains(line, "[error]") {
				decreaseCounter(downloads, name)
			} else {
				increaseCounter(downloads, name)
			}

		} else if exportRegex.MatchString(line) {
			recid := exportRegex.FindStringSubmatch(line)[1]
			increaseCounter(exports, recid)
		}

	}

	// format and print the results
	var formattedStartDate, formattedEndDate string = "N/A", "N/A"
	if !startDate.IsZero() {
		formattedStartDate = startDate.Format(time.RFC1123)
	}
	if !endDate.IsZero() {
		formattedEndDate = endDate.Format(time.RFC1123)
	}

	fmt.Println("Start:", formattedStartDate)
	fmt.Println("End:  ", formattedEndDate)

	// print the number of downloads for each file
	fmt.Println("\nFile downloads:")
	for _, file := range sortedKeys(downloads) {
		count := downloads[file]
		file, _ = url.QueryUnescape(file)
		fmt.Println("\t", file, "was downloaded", count, "times")
	}

	fmt.Println("\nLanding page visits:")
	for _, recid := range sortedKeys(visits) {
		count := visits[recid]
		fmt.Println("\t", recid, "was visited", count, "times")
	}

	fmt.Println("\nMetadata exports:")
	for _, recid := range sortedKeys(exports) {
		count := exports[recid]
		fmt.Println("\t", recid, "was exported", count, "times")
	}
}
