use argparse::{ArgumentParser, Store};
use chrono::NaiveDateTime;
use itertools::Itertools;
use regex::Regex;
use std::collections::HashMap;
use std::io::{self, BufRead};

struct RecordInteraction {
    views: i32,
    exports: i32,
    file_downloads: HashMap<String, i32>,
}

impl RecordInteraction {
    fn zero() -> Self {
        RecordInteraction {
            views: 0,
            exports: 0,
            file_downloads: HashMap::new(),
        }
    }
}

fn parse_datetime(string: &String) -> Option<NaiveDateTime> {
    let ext_string = format!("{} 00:00:00", string);
    match string.as_str() {
        "" => None,
        _ => match NaiveDateTime::parse_from_str(string, "%Y-%m-%d %H:%M:%S") {
            Ok(timestamp) => Some(timestamp),
            Err(_error) => match NaiveDateTime::parse_from_str(&ext_string, "%Y-%m-%d %H:%M:%S") {
                Ok(timestamp) => Some(timestamp),
                Err(_error) => {
                    println!("{}, {}", _error, string);
                    None
                }
            },
        },
    }
}

fn main() {
    let mut min_timestamp_str: String = "".to_string();
    let mut max_timestamp_str: String = "".to_string();
    let mut recid_prefix: String = "".to_string();

    {
        // this block limits the scope of borrows for the parser.refer() methods
        let mut parser = ArgumentParser::new();
        parser.set_description("Nginx log parser for getting InvenioRDM record statistics");
        parser.refer(&mut min_timestamp_str).add_option(
            &["-s", "--start"],
            Store,
            "Start datetime (YYYY-MM-DD HH:MM:SS)",
        );
        parser.refer(&mut max_timestamp_str).add_option(
            &["-e", "--end"],
            Store,
            "End datetime (YYYY-MM-DD HH:MM:SS)",
        );
        parser
            .refer(&mut recid_prefix)
            .add_option(&["-p", "--prefix"], Store, "Recid prefix");
        parser.parse_args_or_exit();
    }

    let min_timestamp = parse_datetime(&min_timestamp_str);
    let max_timestamp = parse_datetime(&max_timestamp_str);

    let file_download_regex =
        Regex::new(r"GET /records/(\w+\-\w+)/files/(.*?)(\?download=1|\?preview=\d|\s)").unwrap();
    let export_regex = Regex::new(r"GET /records/(\w+\-\w+)/export/").unwrap();
    let landing_page_regex = Regex::new(r"GET /records/(\w+\-\w+)\s").unwrap();
    let timestamp_regex =
        Regex::new(r"\[(\d{2}/\w{3}/\d{4}:\d{2}:\d{2}:\d{2}\s+.?\d{0,4})\]").unwrap();

    let mut record_interactions = HashMap::new();
    let mut start_timestamp: Option<NaiveDateTime> = None;
    let mut end_timestamp: Option<NaiveDateTime> = None;

    for line in io::stdin().lock().lines() {
        let line = line.unwrap();

        // check the line for a timestamp and parse it
        let timestamp;
        if timestamp_regex.is_match(&line) {
            let capture = timestamp_regex.captures(&line).unwrap();
            timestamp = NaiveDateTime::parse_from_str(&capture[1], "%d/%b/%Y:%H:%M:%S %z").unwrap();

            // check if the line is within the user-specified bounds
            if let Some(start) = min_timestamp {
                if start > timestamp {
                    continue;
                }
            }
            if let Some(end) = max_timestamp {
                if end < timestamp {
                    continue;
                }
            }

            end_timestamp = Some(timestamp);
            if start_timestamp == None {
                start_timestamp = Some(timestamp);
            }
        } else {
            continue;
        }

        // check which kind of request it was & for which record
        if landing_page_regex.is_match(&line) {
            let cap = landing_page_regex.captures(&line).unwrap();
            let recid = &cap[1];

            let entry = record_interactions
                .entry(String::from(recid))
                .or_insert(RecordInteraction::zero());
            entry.views += 1;
        } else if file_download_regex.is_match(&line) {
            let cap = file_download_regex.captures(&line).unwrap();
            let recid = &cap[1];
            let file = urlencoding::decode(&cap[2]).unwrap();

            let entry = record_interactions
                .entry(String::from(recid))
                .or_insert(RecordInteraction::zero());
            let count = entry.file_downloads.entry(String::from(file)).or_insert(0);
            *count += 1;
        } else if export_regex.is_match(&line) {
            let cap = export_regex.captures(&line).unwrap();
            let recid = &cap[1];

            let entry = record_interactions
                .entry(String::from(recid))
                .or_insert(RecordInteraction::zero());
            entry.exports += 1;
        }
    }

    let start = match start_timestamp {
        Some(timestamp) => timestamp.format("%Y-%m-%d %H:%M:%S").to_string(),
        None => panic!("No proper logs could be parsed!"),
    };
    let end = end_timestamp
        .unwrap()
        .format("%Y-%m-%d %H:%M:%S")
        .to_string();

    println!("Start: {}", start);
    println!("End:   {}", end);
    println!("======={}", "=".repeat(end.len()));

    // print the results
    for recid in record_interactions.keys().sorted() {
        let interactions = &record_interactions[recid];
        println!(
            "\n{}{} - {} views, {} exports",
            recid_prefix, recid, interactions.views, interactions.exports
        );
        for file in interactions.file_downloads.keys().sorted() {
            println!(
                "\t{}: {} downloads",
                file, interactions.file_downloads[file]
            );
        }
    }
}
