#!/usr/bin/env python3

import json
import ssl
import sys
from argparse import ArgumentParser
from http.client import HTTPConnection, HTTPSConnection
from os import environ, listdir, path


def create_connection(host, warn=False):
    """Create an HTTP(S) connection to the specified host."""
    if host.startswith("http://"):
        conn = HTTPConnection(host[7:])
        if warn:
            print("warning: operating via HTTP instead of HTTPS", file=sys.stderr)

    elif host.startswith("https://"):
        conn = HTTPSConnection(host[8:])

    else:
        conn = HTTPSConnection(host)

    return conn


def request(conn, url, method, data=None, headers=None):
    """Perform an HTTP(S) request with the specified parameters.

    Returns a tuple (text, data), where text is the concatenation of all lines in the
    response, and data is the dictionary parsed by JSON.
    """
    headers = headers or {}
    conn.request(method, url, body=data, headers=headers)
    resp = conn.getresponse()
    text = "\n".join([l.decode("utf-8").rstrip("\n") for l in resp])

    if not 200 <= resp.status < 300:
        raise Exception(text)

    data = json.loads(text)
    return text, data


def is_json_file(file_path):
    """Check if the specified file exists and contains JSON data."""
    if not path.isfile(file_path):
        return False

    with open(file_path, "r") as fp:
        try:
            json.load(fp)
        except json.decoder.JSONDecodeError:
            return False

    return True


def merge(*dicts):
    """Merge multiple dictionaries into one."""
    res = {}
    for d in dicts:
        for k in d:
            if k not in res:
                res[k] = d[k]

    return res


parser = ArgumentParser(
    description="upload zero or more records (either with files, or metadata-only) to an InvenioRDM instance."
)
parser.add_argument(
    "-P",
    "--do-not-publish",
    action="store_true",
    default=False,
    help="do not publish the created records -- instead, keep them in an editable draft state",
)
parser.add_argument(
    "-H",
    "--host",
    default="localhost:5000",
    help="the hostname (with port) of the target InvenioRDM instance (default: 'https://localhost:5000')",
)
parser.add_argument(
    "-t",
    "--access-token",
    default=environ.get("ACCESS_TOKEN"),
    help="the personal access token for the REST API",
)
parser.add_argument(
    "-k",
    "--insecure",
    action="store_true",
    default=False,
    help="skip SSL validation, e.g. for allowing self-signed certificates (not generally recommended)",
)
parser.add_argument(
    "record",
    nargs="*",
    help=(
        "either a JSON file with the metadata for a metadata-only record, "
        "or a directory containing a 'metadata.json' file and optionally a "
        "'files/' subdirectory with files to upload. can be specified multiple times."
    ),
)

args = parser.parse_args()
publish = not args.do_not_publish
records = args.record
host = args.host
access_token = args.access_token
if not access_token:
    # the access token is a hard requirement
    print("error: ACCESS_TOKEN required but not specified", file=sys.stderr)
    sys.exit(1)

if args.insecure:
    # if specified, allow unverified SSL stuff
    ssl._create_default_https_context = ssl._create_unverified_context

# different headers for the requests
json_header = {"Content-Type": "application/json"}
octet_header = {"Content-Type": "application/octet-stream"}
auth_header = {"Authorization": "Bearer {}".format(access_token)}
auth_json_header = merge(auth_header, json_header)
auth_octet_header = merge(auth_header, octet_header)

for record in records:
    # each item in the "records" list should either point to a JSON file (for
    # metadata-only records), or a directory containing a "metadata.json" file and
    # optionally a "files/" subdirectory with the files to upload
    metadata_file = path.join(record, "metadata.json")
    files_dir = path.join(record, "files")
    file_names = []

    # see which it is (metadata-only, or record with files)
    if is_json_file(record):
        metadata_file = record

    elif path.isdir(record) and is_json_file(metadata_file):
        file_names = [
            fn for fn in listdir(files_dir) if path.isfile(path.join(files_dir, fn))
        ]

    step = "create record"
    try:
        step = "upload metadata"
        with open(metadata_file, "r") as fp:
            content = "\n".join([l.rstrip("\n") for l in fp.readlines()])
            text, data = request(
                create_connection(host, warn=True),
                "/api/records",
                "POST",
                data=content,
                headers=auth_json_header,
            )

            # errors in the metadata are reported via the "errors" field in the response
            if data and "errors" in data:
                print(
                    "errors from response at step '{}' while processing '{}':".format(
                        step, record
                    ),
                    file=sys.stderr,
                )
                print(data["errors"], file=sys.stderr)

            recid = data["id"]

        # if there are any files to be uploaded, invenio needs to know about them
        # (maybe this could be done in separate requests, but one is enough)
        if file_names:
            step = "initialize files"
            keys = ['{"key":"%s"}' % fn for fn in file_names]
            file_key_list = "[{}]".format(",".join(keys))
            request(
                create_connection(host),
                "/api/records/{}/draft/files".format(recid),
                "POST",
                data=file_key_list,
                headers=auth_json_header,
            )

        # upload each file (first PUT the file content, then POST a commit)
        for file_name in file_names:
            step = "upload file ({})".format(file_name)
            file_path = path.join(files_dir, file_name)
            with open(file_path, "rb") as fp:
                request(
                    create_connection(host),
                    "/api/records/{}/draft/files/{}/content".format(recid, file_name),
                    "PUT",
                    data=fp.read(),
                    headers=auth_octet_header,
                )
                request(
                    create_connection(host),
                    "/api/records/{}/draft/files/{}/commit".format(recid, file_name),
                    "POST",
                    headers=auth_header,
                )

        # if not otherwise specified, publish the record
        step = "publish record"
        if publish:
            request(
                create_connection(host),
                "/api/records/{}/draft/actions/publish".format(recid),
                "POST",
                headers=auth_header,
            )

        print(recid)

    except Exception as e:
        # in case of an error, print the error and skip to the next record
        print(
            "error ({}) at step '{}' while processing '{}':".format(
                type(e), step, record
            ),
            file=sys.stderr,
        )
        print(e, file=sys.stderr)
        continue
